from . import db, ModelBase, View
from backend.utils.misc import ArrayType
import enum
from sqlalchemy import Enum
from typing import List


class UrlRuleMode(str, enum.Enum):
    DENY = "DENY"
    ALLOW = "ALLOW"


class UrlRuleMatchingType(str, enum.Enum):
    CONTAINS = "CONTAINS"
    MATCHES = "MATCHES"
    STARTS_WITH = "STARTS_WITH"
    EQUALS = "EQUALS"


class UrlRule(ModelBase):
    __tablename_base__ = "url_rule"

    id: int = db.Column(db.Integer, primary_key=True, autoincrement=True)
    view_id: int = db.Column(db.Integer, db.ForeignKey(View.id, ondelete="CASCADE"), nullable=False)

    matching_type: UrlRuleMatchingType = db.Column(Enum(UrlRuleMatchingType), nullable=False)
    mode: UrlRuleMode = db.Column(Enum(UrlRuleMode), nullable=False)
    priority: int = db.Column(db.Integer, nullable=False)
    value: str = db.Column(db.String(64), nullable=True)
    css_selectors: List[str] = db.Column(ArrayType(), nullable=True, default=[])
