import logging

from flask import Blueprint, jsonify, request
from backend.utils.dataiku_api import dataiku_api, NO_TRACKING_CONFIG
from backend.controllers import auth_ctrl

utils = Blueprint("utils", __name__)
logger = logging.getLogger(__name__)


@utils.route("/ping")
def ping():
    return jsonify({"message": "pong"})


@utils.route("/secured-ping")
def secured_ping():
    return jsonify({"message": "secured-pong"})


@utils.route("/offline-data", methods=["POST"])
def offline_data():
    body = request.get_json()
    return jsonify({"data": body})


@utils.route("/log", methods=["POST"])
def log():
    body = request.get_json()
    if len(body["data"]) == 1:
        data = body["data"][0]
    else:
        data = body["data"]
    logger.info(f"🐦 {body['logger']} : {data}")
    return jsonify("ok")


@utils.route("/config")
def config():
    current_user = auth_ctrl.get_current_dss_user(request)
    return jsonify({"user": {"id": current_user.login, "email": current_user.email}})


@utils.route("/tracking-config")
def tracking_config():
    headers = dict(request.headers)
    try:
        tracking_config = dataiku_api.get_tracking_config(headers)
    except Exception as err:
        logger.error("Something went wrong fetching the tracking config. Error: ", err)
        return NO_TRACKING_CONFIG
    return {"tracking_config": tracking_config}
