from faker import Faker
import random as rd

from backend.models import db
from backend.models import View, Connection, UrlRule, Card
from backend.models.connection import MultipleMatchesMode
from backend.models.url_rule import UrlRuleMode, UrlRuleMatchingType
from backend.models.card import CardType
from backend.utils.dataiku_api import dataiku_api


fake = Faker()

TEST_DSS_HOST = "http://localhost:25200/"
TEST_DSS_API_KEY = "WsVm4o6gkJODEN6tbtdYIfCylQXxTFY6"

TEST_PROJECT_KEY = "ADVISORTEST"
TEST_DATASET_NAME = "accounts_infos_for_advisor_copy_prepared_copy"
TEST_DATASET_FULL_NAME = f"{TEST_PROJECT_KEY}.{TEST_DATASET_NAME}"
TEST_KEYWORDS_COLUMN_NAME= "Account Name"
TEST_KEYWORD = "pfizer"



class EndpointTest:
    endpoint_tested = {}
    endpoint_skipped = {}

    @staticmethod
    def test_for_endpoint(endpoint, methods):
        EndpointTest.endpoint_tested[endpoint] = EndpointTest.endpoint_tested.get(endpoint, set()).union(
            {_rm for _rm in methods if _rm not in ["HEAD", "OPTIONS"]}
        )

    @staticmethod
    def skip_for_endpoint(endpoint, methods, reason):
        EndpointTest.test_for_endpoint(endpoint, methods)
        for method in methods:
            EndpointTest.endpoint_skipped[(endpoint, method)] = reason

def gen_fake_view(**kwargs):
    new_view = View(
        name=kwargs.get("name", fake.sentence(nb_words=2)),
        is_active=kwargs.get("is_active", fake.pybool()),
    )
    return new_view

def gen_fake_connection(view_id, **kwargs):
    return Connection(
        view_id=view_id,
        dataset_name=TEST_DATASET_FULL_NAME,
        keywords_column_name=TEST_KEYWORDS_COLUMN_NAME,
        multiple_matches_mode=MultipleMatchesMode.KEEP_FIRST,
        secondary_column_name=None
    )

def gen_fake_url_rule(view_id, **kwargs):
    return UrlRule(
        view_id=view_id,
        matching_type=kwargs.get("matching_type", rd.choice(list(UrlRuleMatchingType))),
        mode=kwargs.get("mode", rd.choice(list(UrlRuleMode))),
        priority=kwargs.get("priority", rd.randint(1, 100)),
        value=kwargs.get("value", fake.word()),
        css_selectors=kwargs.get("value", [fake.word() for i in range(rd.randint(0, 5))])
    )

def gen_fake_card(view_id, **kwargs):
    test_dataset_columns = dataiku_api.list_columns(TEST_DATASET_NAME, TEST_PROJECT_KEY)
    test_dataset_columns_name = [c["name"] for c in test_dataset_columns]
    return Card(
        view_id=view_id,
        name=kwargs.get("name", fake.sentence(nb_words=2)),
        type=kwargs.get("type", rd.choice(list(CardType))),
        data_columns=kwargs.get("data_columns", rd.sample(test_dataset_columns_name, 1)),
        priority=kwargs.get("priority", rd.randint(1, 100)),
        test_keyword=TEST_KEYWORD
    )


def insert_fake_view(**kwargs):
    new_view = gen_fake_view(**kwargs)
    db.session.add(new_view)
    db.session.commit()
    return new_view

def insert_fake_connection(view_id, **kwargs):
    new_connection = gen_fake_connection(view_id, **kwargs)
    db.session.add(new_connection)
    db.session.commit()
    return new_connection

def insert_fake_url_rule(view_id, **kwargs):
    new_url_rule = gen_fake_url_rule(view_id, **kwargs)
    db.session.add(new_url_rule)
    db.session.commit()
    return new_url_rule

def insert_fake_card(view_id, **kwargs):
    new_card = gen_fake_card(view_id, **kwargs)
    db.session.add(new_card)
    db.session.commit()
    return new_card

def insert_many_url_rules(view_id, n, **kwargs):
    rules_set = []
    for i in range(n):
        new_rule = insert_fake_url_rule(view_id, **kwargs)
        rules_set.append(new_rule)
    return rules_set
