from typing import Optional
from dateutil import parser as date_iso_parser
import csv
from contextlib import closing
import itertools
from typing import Dict, List

import codecs

dku_basestring_type = str
dku_zip_longest = itertools.zip_longest


def none_if_throws(f):
    def aux(*args, **kargs):
        try:
            return f(*args, **kargs)
        except Exception as err:
            return None

    return aux


class DataikuStreamedHttpUTF8CSVReader(object):
    """
    A CSV reader with a schema
    """

    def __init__(self, csv_stream, schema: List[Dict], chunk_size: Optional[int] = None):
        self.schema = schema
        self.csv_stream = csv_stream
        self.chunk_size = chunk_size

    def iter_rows(self, chunk_index: int = 0):
        def decode(x):
            return x

        def parse_iso_date(s):
            if s == "":
                return None
            else:
                return date_iso_parser.parse(s)

        def str_to_bool(s):
            if s is None:
                return False
            return s.lower() == "true"

        CASTERS = {
            "tinyint": int,
            "smallint": int,
            "int": int,
            "bigint": int,
            "float": float,
            "double": float,
            "date": parse_iso_date,
            "boolean": str_to_bool,
        }
        casters = [CASTERS.get(col["type"], decode) for col in self.schema]
        with closing(self.csv_stream) as r:
            raw_generator = codecs.iterdecode(r.raw, "utf-8")
            chunk = []
            for i, uncasted_tuple in enumerate(
                csv.reader(raw_generator, delimiter="\t", quotechar='"', doublequote=True)
            ):

                if self.chunk_size is not None and i < chunk_index * self.chunk_size:
                    continue

                row = [none_if_throws(caster)(val) for (caster, val) in dku_zip_longest(casters, uncasted_tuple)]

                chunk.append(row)

                if self.chunk_size is not None and len(chunk) > self.chunk_size:
                    yield chunk
                    chunk = []

            if len(chunk):
                yield chunk
